class DescriptionHelper:
    
    def summary_param_bank(self):
        '''
        Method contains summarizing descriptions of general parameters.
        '''

        time_domain_summary_param = [
        'Przedstawione wyniki pokazują wpływ zmian badanego parametru \({param_name}\) na dynamikę wózka. ',
        'Opracowane wykresy porównawcze pozwoliły na oszacowanie wpływu zmiennej \({param_name}\) na dynamikę rozpatrywanego układu. ',
        'Zaprezentowane wyniki posłużyły ocenie wrażliwości modelowanego obiektu na badaną zmienną \({param_name}\). ',
        'Przygotowane przebiegi służą zaprezentowaniu wyników symulacji numerycznych badanej zmiennej w funkcji wybranego parametru wpływowego \({param_name}\). ',
        'Analizowany model wykazuje wrażliwość na zmianę rozważanego parametru \({param_name}\). ',
        ]
        time_domain_summary_x = [
        'Zaobserwowano wpływ badanego parametru \({x(t)}\) na drgania wózka, których maksymalna wartość amplitudy wyniosła {x(t)_max}. ',
        'Analizowany przebieg parametru \({x(t)}\) osiąga wartość maksymalną równą {x(t)_max}. ',
        'Minimalna wartość dla przebiegu \({x(t)}\) jest równa {x(t)_min}. ',
        'Na podstawie przeprowadzonych badań numerycznych można stwierdzić, że w analizowanym przebiegu parametru \({x(t)}\) nie występują amplitudy o wartości mniejszej niż {x(t)_min} oraz  większej niż {x(t)_max}. ',
        'Dokonując analizy wyników symulacji dla parametru \({x(t)}\) stwierdza się, że amplitudy drgań nie przekraczają wartości {x(t)_max}, a wartość mininalna wynosi {x(t)_min}. ',
        ]

        return self
        
    def summary_var_bank(self):
        '''
        Method include summarizing descriptions of generalized variables.
        '''
        
        time_domain_summary_z = [ 
        'Podsumowując symulację możemy określić, że wartość maksymalna dla współrzędnej uogólnionej \({z(t)}\) jest równa {z(t)_max}. ',
        'Poddawane analizie sygnały cechują się wartością maksymalną równą {z(t)_max} dla współrzędnej \({z(t)}\). ',
        'Przebieg współrzędnej \({z(t)}\) charakteryzuje się wartością minimalną równą {z(t)_min}. ',
        'Wykonane badania numeryczne umożliwiają stwierdzenie, że odpowiedź czasowa \({z(t)}\) nie przekracza {z(t)_max} i jest ograniczana z dołu przez {z(t)_min}. ',
        'Opierając się na analizie przeprowadzonych badań stwierdzono, że współrzędna \({z(t)}\) nie przekracza wartości {z(t)_max}, a jej wartość mininalna to {z(t)_min}. ',
        ]
        time_domain_summary_z_wrc = [ 
        'Przedłożone wyniki symulacji pokazują, że wartość maksymalna dla współrzędnej uogólnionej \({z_wrc(t)}\) wynosi {z_wrc(t)_max}. ',
        'Analizowane sygnały reprezentujące odpowiedż czasową układu charakteryzują się wartością maksymalną równą {z_wrc(t)_max} dla współrzędnej \({z_wrc(t)}\). ',
        'Wartość minimalna przebiegu współrzędnej \({z_wrc(t)}\) jest równa {z_wrc(t)_min}. ',
        'Przeprowadzone badania numeryczne pozwalają stwierdzić, że odpowiedź czasowa \({z_wrc(t)}\) nie przekracza {z_wrc(t)_max} i jest ograniczona z dołu przez {z_wrc(t)_min}. ',
        'Po dokonaniu analizy badań zaobserwowano, że współrzędna \({z_wrc(t)}\) nie przekracza wartości {z_wrc(t)_max}, a wartość mininalna wynosi {z_wrc(t)_min}. ',
        ]
        time_domain_summary_z_r = [ 
        'Wyniki symulacji ukazują, że wartość maksymalna dla współrzędnej uogólnionej \({z_r(t)}\) wynosi {z_r(t)_max}. ',
        'Analizowane sygnały reprezentujące odpowiedż czasową układu charakteryzują się wartością maksymalną równą {z_r(t)_max} dla współrzędnej \({z_r(t)}\). ',
        'Wartość minimalna przebiegu współrzędnej \({z_r(t)}\) jest równa {z_r(t)_min}. ',
        'Przeprowadzone badania numeryczne pozwalają stwierdzić, że odpowiedź czasowa \({z_r(t)}\) nie przekracza {z_r(t)_max} i jest ograniczona z dołu przez {z_r(t)_min}. ',
        'Na podstawie analizy przeprowadzonych badań stwierdzono, że współrzędna \({z_r(t)}\) nie przekracza wartości {z_r(t)_max}, a wartość mininalna wynosi {z_r(t)_min}. ',
        ]
        time_domain_summary_z_fr = [ 
        'Rezultaty symulacji pokazują wartość maksymalną {z_fr(t)_max} dla współrzędnej uogólnionej \({z_fr(t)}\). ',
        'Analizowane sygnały reprezentujące odpowiedż czasową układu charakteryzują się wartością maksymalną równą {z_fr(t)_max} dla współrzędnej \({z_fr(t)}\). ',
        'Wartość minimalna przebiegu współrzędnej \({z_fr(t)}\) jest równa {z_fr(t)_min}. ',
        'Przeprowadzone badania numeryczne pozwalają określić, że odpowiedź czasowa \({z_fr(t)}\) nie przekracza {z_fr(t)_max} i jest ograniczana z dołu przez {z_fr(t)_min}. ',
        'Poprzez analizie przeprowadzonych badań zaobserwowano, że współrzędna \({z_fr(t)}\) nie przekracza wartości {z_fr(t)_max}, a wartość mininalna wynosi {z_fr(t)_min}. ',
        ]
        
        return self
    
    def summary_angle_bank(self):
        '''
        Method includes summarizing descriptions of examined angles.
        '''
        time_domain_summary_varphi = [ 
        'Wyniki symulacji prezentują wartość maksymalną {varphi(t)_max} dla współrzędnej uogólnionej \({varphi(t)}\). ',
        'Analizowane sygnały reprezentujące informacje zwrotną czasową układu charakteryzują się wartością maksymalną równą {varphi(t)_max} dla współrzędnej \({varphi(t)}\). ',
        'Wartość minimalna przebiegu współrzędnej \({varphi(t)}\) wynosi {varphi(t)_min}. ',
        'Przeprowadzone badania numeryczne pozwalają stwierdzić, że odpowiedź czasowa \({varphi(t)}\) nie przekracza {varphi(t)_max} i jest ograniczona z dołu przez {varphi(t)_min}. ',
        'Analiza przeprowadzonych badań umożliwiła wyciągnięcie wniosków, że w przypadku współrzędnej \({varphi(t)}\) nie dochodzi do przekroczenia wartości {varphi(t)_max}, a wartość     mininalna wynosi {varphi(t)_min}. ',
        ]
        time_domain_summary_theta = [ 
        'Rezultaty symulacji wykazują, że wartość maksymalna dla współrzędnej uogólnionej \({theta(t)}\) osiąga wartość {theta(t)_max}. ',
        'Weryfikowane sygnały reprezentujące rezultat czasowy układu wyróżniają się wartością maksymalną równą {theta(t)_max} dla współrzędnej \({theta(t)}\). ',
        'Przebieg współrzędnej \({theta(t)}\) cechuje się wartością minimalną {theta(t)_min}. ',
        'Badania numeryczne pozwalają wywnioskować, że informacja zwrotna czasowa \({theta(t)}\) nie przekracza {theta(t)_max} oraz jest ograniczona z dołu przez {theta(t)_min}. ',
        'Biorąc pod uwagę przeprowadznone badania można zaobserwować, że współrzędna \({theta(t)}\) nie przekracza wartości {theta(t)_max}, a wartość mininalna wynosi {theta(t)_min}. ',
        ]
        time_domain_summary_varphi_RC = [ 
        'Wyniki symulacji pokazują, że wartość maksymalna dla współrzędnej uogólnionej \({varphi_RC(t)}\) jest równa {varphi_RC(t)_max}. ',
        'Analizowane sygnały charakteryzują się wartością maksymalną równą {varphi_RC(t)_max} dla współrzędnej \({varphi_RC(t)}\).',
        'Wartość minimalna przebiegu współrzędnej \({varphi_RC(t)}\) jest określona wartością {varphi_RC(t)_min}. ',
        'Odpowiedź czasowa \({varphi_RC(t)}\) nie przekracza {varphi_RC(t)_max} i jest ograniczana z dołu przez {varphi_RC(t)_min}, co pozwala nam stwierdzić analiza przeprowadzonych badań. ',
        'Bazując na analizie przeprowadzonych badań zaobserwowano, że współrzędna \({varphi_RC(t)}\) nie przekracza wartości {varphi_RC(t)_max}, a wartość mininalna wynosi {varphi_RC(t)_min}. ',
        ]
        
        return self
    
    def general_summary(self):
        '''
        Method includes general concluding sentences.
        '''
        # to improve
        time_domain_summary_tval = [
        'Zaobserwowane odczyty wynikające z wpływu \({param_name}\) występują w konketrnym czasie ruchu {t_val}. ',
        'Odczytana maksymalna wartość amplitudy {max_val} wynika z działania \({param_name}\) wystąpiła konkretnie w czasie {t_val}. '
        'Minimalny zaobserwowany wynik amplitudy {min_val} miał miejsce w {t_val} czasu trwania pomiaru. ',
        ]


        freq_domain_summary =[
        'Ponadto sprawdzono własności spektralne otrzymanych wyników symulacyjnych.',
        'Według przeprowadzonych badań modelowych struktura częstotliwościowa nie zawiera komonentów większych niż {max_val_spec}.',
        'Amplitudy drgań badanego parametru \({q_name}\) przyjmują maksymalną wartość równą {max_val_spec}.',
        'Na podstawie analizy widmowej minimalna wartość częstotliwości rozpatrywanego parametru wynosi {min_val_spec}.',


        ]

        summary_bank_optimization=[
        'Zaprezentowano efekty symulacji optymalizacyjnych dla badanego modelu dynamicznego wózka inwalidzkiego wraz z zamontowanym napędem RC. Dla uzyskanych danych symulacyjnych przygotowano wykresy przedstawiające funkcję celu w funkcji analizowanego parametru. Analizowane dane symulacyjne pozwalają na określenie wartości minimalnej'
        ]
        
        return self
    
    def sec_introduction_gen(self):
        '''
        Method contains sentences describing process of research.
        '''
        sec_introduction_bank_gen1=[
        'Wykorzystując przygotowany model dynamiczny, przeprowadzono serię symulacji mających na celu ocenę wrażliwości modelu na zmiany wybranych parametrów. ',
        'Przeprowadzenie analizy numerycznej wymaga znajomości modelu dynamicznego oraz zdeterminowania parametrów oddziałujących na zachowanie układu wskutek ich zmian. ',
        'Wykorzystując przygotowane środowisko obliczeniowe, wykonano szereg symulacji, opracowano dane numeryczne i przedstawiono je w postaci następujących wykresów. ',
        'Celem zbadania i oceny dynamiki rozpatrywanego systemu przeprowadzono symulacje numeryczne przedstawijące przebiegi zmiennych układu w funkcji czasu. '
        'W oparciu o wybrany model dynamiczny wózka, wykonano analizy numeryczne zaprezentowane na kolejnych wykresach. '
        ]

        sec_introduction_bank_gen2=[
        'Na podstawie spostrzeżeń wynikających z obserwacji odpowiedzi czasowych analizowanego układu możliwe będzie wprowadzenie zmian, które wpłyną na poprawę działania modelu. ',
        'Charakter zmian przebiegów poszczególnch współrzędnych może posłużyć do opracowania wniosków, na podstawie których możliwe będzie ulepszenie funkcjonowania rozważanego układu. ',
        'Zaprezentowane wykresy przedstawiające odpowiedzi czasowe układu maja na celu ułatwienie analizy zachowania modelu dynamicznego oraz wprowadzenie ewentualnych poprawek na podstawie dokonanych obserwacji i spostrzeżeń. ',
        'Dostrzegając wzajemne zależności między dynamicznymi parametrami wózka możliwe będzie wprowadzenie takich zmian, które w korzystny sposób wpłyną na odpowiedź układu. '
        ]


        sec_introduction_bank_gen3=[
        'Na podstawie spostrzeżeń wynikających z obserwacji odpowiedzi czasowych analizowanego układu możliwe będzie wprowadzenie zmian, które wpłyną na poprawę działania modelu. ',
        'Charakter zmian przebiegów poszczególnch współrzędnych może posłużyć do opracowania wniosków, na podstawie których możliwe będzie ulepszenie funkcjonowania rozważanego układu. ',
        'Zaprezentowane wykresy przedstawiające odpowiedzi czasowe układu maja na celu ułatwienie analizy zachowania modelu dynamicznego oraz wprowadzenie ewentualnych poprawek na podstawie dokonanych obserwacji i spostrzeżeń. ',
        'Dostrzegając wzajemne zależności między dynamicznymi parametrami wózka możliwe będzie wprowadzenie takich zmian, które w korzystny sposób wpłyną na odpowiedź układu. '
        ]
        
        return self
    
    def introduction_gen(self):
        '''
        Method includes sentences describing received data and graphs.
        '''
        
        introduction_bank_fig_no=[
        'Na wykresie {nr_rys} przedstawiano zmiany wielkości dynamicznych charakteryzujących ruch wózka. Po przeanalizowaniu można zanotować wzajemną zależność poszczególnych wielkości dynamicznych.',
        'Charakter przebiegu wartości dynamicznych wózka został przedstawiony na rysunku {nr_rys}.',
        'Wykres {nr_rys} pokazuje zmienność parametrów dynamicznych wózka w trakcie symulownego przejazdu.',
        'Rysunek {nr_rys} prezentuje wykres opisujący zmienność parametrów dynamicznych wózka w trakcie jazdy po zamodelowanym torze.',
        ]

        introduction_bank_given_data=[
        'Symulacja została przeprowadzona dla następujących danych: {given_data}.',
        'Zaprezentowane wyniki numeryczne otrzymano dla danych równych:{given_data}.',
        'Eksperyment numeryczny został przeprowadzona dla następujących danych:{given_data}.',
        'Wyniki stmulacji numerycznych otrzymano dla:{given_data}.',
        ]
        introduction_bank_gen1=[
        'Na podstawie spostrzeżeń wynikających z obserwacji odpowiedzi czasowych analizowanego układu stwierdza się oscylacyjny charakter odpowiedzi układu. Dynamika systemu ma stabilny charakter',
        'Charakter zmian przebiegów poszczególnch współrzędnych jest stabilny. Otrzymane sygnały mieszczą się w zakresie dopuszczalnych uwarunkowaniami fizycznymi.',
        'Zaprezentowane wykresy przedstawiające odpowiedzi czasowe układu mają drganiowy charakter i są zgodne z oczekiwaniami.',
        'Bazując na otrzymanych wynikach można stwierdzić wzajemną zależność poszczególnych wielkości dynamicznych. Dodatkow wielkości modelu stabilizują się w czasie. ',
        ]
        
        return self
    
    def conclusion_param_bank(self):
        '''
        Method contains parameters' concluding descriptions.
        '''
        conclusion_bank_x=['Zauważa się {x(t)_inf} zmienności parametru \({param_name}\) dla współrzędnej \({x(t)}\) oraz odpowiadającej temu przemieszczniu prędkości. Stwierdzono wpływ badanego parametru, gdzie maksymalne wartości dla wymienionych współrzędnych przyjmują odpowiednio {x(t)_max} oraz {Derivative(x(t), t)_max} dla {x(t)_idmax} i {Derivative(x(t), t)_idmax}. ',
                           
        'Zaobserwowano {x(t)_inf} parametru \({param_name}\) na maksymalną wartość pokonanej drogi oraz osiąganą wartość prędkości. Maksymalne wartości dla wymienionych współrzędnych przyjmują odpowiednio {x(t)_max} oraz {Derivative(x(t), t)_max} dla maksymalnej wartości badanego parametru równej {x(t)_idmax}. ',
                           
        'Zmianę dynamiki pod wpływem zmienności parametru \({param_name}\) obserwuje się dla \({x(t)}\), gdzie największa wartość pokonanej drogi to {x(t)_max}. W konsekwencji zaobserwowano {x(t)_inf} analizowanej zmiennej na wartość prędkości liniowej \({Derivative(x(t), t)}\), dla której minimalna wartość wynosi {Derivative(x(t), t)_min}, a największą osiąganą wartością jest {Derivative(x(t), t)_max} odpowiednio dla wartości parmametru: {Derivative(x(t), t)_idmin} oraz {Derivative(x(t), t)_idmax}. ',
                  ]
        
        return self
    
    def conclusion_var_bank(self):
        '''
        Method contains variables' concluding descriptions.
        '''
        conclusion_bank_z=['Zmianę dynamiki pod wpływem zmienności parametru \({param_name}\) obserwuje się dla \({z(t)}\), gdzie największa wartość amplitudy to {z(t)_max}, a najmniejsza {z(t)_min}. W konsekwencji zaobserwowano {z(t)_inf} analizowanej zmiennej na wartość drgań prędkości drgań pionowych, dla których minimalna wartość amplitudy to {Derivative(z(t), t)_min}, a największą osiąganą wartością jest {Derivative(z(t), t)_max} odpowiednio dla wartości parmametru: {Derivative(z(t), t)_idmin} oraz {Derivative(z(t), t)_idmax}. ',

        'Zauważa się {z(t)_inf} zmienności parametru \({param_name}\) dla współrzędnej \({z(t)}\) oraz odpowiadającej temu przemieszczniu prędkości. Stwierdzono wpływ badanego parametru, gdzie maksymalne wartości dla wymienionych współrzędnych przyjmują odpowiednio {z(t)_max} oraz {Derivative(z(t), t)_max} dla {z(t)_idmax} i {Derivative(z(t), t)_idmax}. '

        'Zaobserwowano {z(t)_inf} parametru \({param_name}\) na maksymalną wartość drgań pionowych wózka oraz osiąganą wartość ich prędkości. Maksymalne wartości dla wymienionych współrzędnych przyjmują odpowiednio {z(t)_max} oraz {Derivative(z(t), t)_max} dla maksymalnej wartości badanego parametru równej {z(t)_idmax}. '
                ]
        
        return self
    
    def conclusion_angle_bank(self):
        '''
        Method contains angles' concluding descriptions.
        '''
        
        
        conclusion_bank_varphi_rc=['Zaobserwowano {varphi_RC(t)_inf} rozpatrywanego parametru - \({param_name}\) na wartość drgań i prędkość kątową napędu RC. Przemieszczenia kątowe nie przyjmują wartości mniejszej niż {varphi_RC(t)_min} oraz większej niż {varphi_RC(t)_max} odpowiednio dla wartości parametru: {varphi_RC(t)_idmin} oraz {varphi_RC(t)_idmax}. Dla prędkości kątowej napędu minimalna wartość amplitudy to {Derivative(varphi_RC(t), t)_min}, a największą osiąganą wartością jest {Derivative(varphi_RC(t), t)_max}.' ,

        'Zmiana \({param_name}\) ma {varphi_RC(t)_inf} na wartość drgań i prędkość kątową napędu. Przemieszczenia kątowe nie przyjmują wartości mniejszej niż {varphi_RC(t)_min} oraz większej niż {varphi_RC(t)_max} odpowiednio dla wartości parametru: {varphi_RC(t)_idmin} oraz {varphi_RC(t)_idmax}. Dla prędkości kątowej napędu minimalna wartość amplitudy to {Derivative(varphi_RC(t), t)_min}, a największą osiąganą wartością jest {Derivative(varphi_RC(t), t)_min}. ',

        'Zmianę dynamiki pod wpływem zmienności parametru \({param_name}\) obserwuje się dla \({varphi_RC(t)}\), gdzie największa wartość amplitudy to {varphi_RC(t)_max}, a najmniejsza {varphi_RC(t)_min}. W konsekwencji zaobserwowano {varphi_RC(t)_inf} analizowanej zmiennej na wartość prędkości kątową napędu RC. Dla prędkości kątowej napędu minimalna wartość amplitudy to {Derivative(varphi_RC(t), t)_min}, a największą osiąganą wartością jest {Derivative(varphi_RC(t), t)_max} odpowiednio dla wartości parmametru: {Derivative(varphi_RC(t), t)_idmin} oraz {Derivative(varphi_RC(t), t)_idmax}. ' ,
                ]

        conclusion_bank_varphi_zwrc=['Zaobserwowano {z_wrc(t)_inf} rozpatrywanego parametru - \({param_name}\) na wartość drgań pionowych i prędkość napędu RC. Przemieszczenia pionowe nie przyjmują wartości mniejszej niż {z_wrc(t)_min} oraz większej niż {z_wrc(t)_max} odpowiednio dla wartości parametru: {z_wrc(t)_idmin} oraz {z_wrc(t)_idmax}. Dla prędkości drgań pionowych napędu minimalna wartość amplitudy to {Derivative(z_wrc(t), t)_min}, a największą osiąganą wartością jest {Derivative(z_wrc(t), t)_max}. ',

        'Zmiana \({param_name}\) ma {z_wrc(t)_inf} na wartość drgań i prędkość kątową analizowanego układu. Przemieszczenia kątowe nie przyjmują wartości mniejszej niż {z_wrc(t)_min} oraz większej niż {z_wrc(t)_max} odpowiednio dla wartości parametru: {z_wrc(z_wrct)_idmin} oraz {z_wrc(t)_idmax}. Dla prędkości kątowej napędu minimalna wartość amplitudy to {Derivative(z_wrc(t), t)_min}, a największą osiąganą wartością jest {Derivative(z_wrc(t), t)_min}. ',

        'Zmianę dynamiki pod wpływem zmienności parametru \({param_name}\) obserwuje się dla \({varphi(t)}\), gdzie największa wartość amplitudy to {varphi_RC(t)_max}, a najmniejsza {varphi(t)_min}. W konsekwencji zaobserwowano {varphi(t)_inf} analizowanej zmiennej na wartość prędkości drgań kątowych badanego układu, dla którego minimalna wartość amplitudy wynosi {Derivative(varphi(t), t)_min}, a największą osiąganą wartością jest {Derivative(varphi(t), t)_max} odpowiednio dla wartości parmametru: {Derivative(varphi(t), t)_idmin} oraz {Derivative(varphi(t), t)_idmax}. '
                ]

        conclusion_bank_phi=['Zmiana \({param_name}\) ma {varphi(t)_inf} na wartość drgań i prędkość kątową układu. Przemieszczenia kątowe nie przyjmują wartości mniejszej niż {varphi(t)_min} oraz większej niż {varphi(t)_max} odpowiednio dla wartości parametru: {varphi(t)_idmin} oraz {varphi(t)_idmax}. Dla prędkości kątowej wózka minimalna wartość amplitudy to {Derivative(varphi(t), t)_min}, a największą osiąganą wartością jest {Derivative(varphi(t), t)_min}. ',
    
        'Zaobserwowano {varphi(t)_inf} rozpatrywanego parametru - \({param_name}\) na wartość drgań i prędkość kątową układu. Przemieszczenia kątowe nie przyjmują wartości mniejszej niż {varphi(t)_min} oraz większej niż {varphi(t)_max} odpowiednio dla wartości parametru: {varphi(t)_idmin} oraz {varphi(t)_idmax}. Dla prędkości kątowej napędu minimalna wartość amplitudy to {Derivative(varphi(t), t)_min}, a największą osiąganą wartością jest {Derivative(varphi(t), t)_max}. ',

        'Zauważa się {varphi(t)_inf} zmienności parametru \({param_name}\) dla współrzędnej \({varphi(t)}\) oraz odpowiadającej temu przemieszczniu kątowemu prędkości. Stwierdzono wpływ badanego parametru, gdzie maksymalne wartości dla wymienionych współrzędnych przyjmują odpowiednio {varphi(t)_max} oraz {Derivative(varphi(t), t)_max} dla {varphi(t)_idmax} i {Derivative(varphi(t), t)_idmax}. ',
                ]
        
        return self
    
    def no_impact_conclusion_bank(self):
        '''
        Method contains sentences describing no impact on any different variables.
        '''
        conclusion_bank_no_impact=['Nie odmotowano natomiast istotnego wpływu tego parametru na pozostałe stopnie swobody układu. ',
                           'Nie zauważono kluczowych zmian pozostałych zmiennych układu pod wpływem badanych wartości rozpatrywanego parametru. ',
                           'Pozostałe zmienne układu nie wykazują wpływu oddziałującego w sposób istoty na dynamikę rozpatrywanego systemu. ',
                           'Badany system dynamiczny nie wykazuje szczególnej wrażliwości na zmiany innych analizowanych parametrów. ',
                ]
        
        return self
    
    def sensitivity_anal_dict(self):
        '''
        Method contains dictionary with described parameters connected with sensitivity.
        '''
        sensitivity_anal_dict = {
                        k_fix:'Zaobserwowano wpływ rozpatrywanego parametru na wartość drgań i prędkość kątową napędu RC. Przemieszczenia kątowe nie przyjmują wartości mniejszej niż {varphi_RC(t)_min} oraz większej niż {varphi_RC(t)_max}. Dla prędkości kątowej napędu minimalna wartość amplitudy to {Derivative(varphi_RC(t), t)_min}, a największą osiąganą wartością jest {Derivative(varphi_RC(t), t)_max}. Nie odmotowano natomiast istotnego wpływu tego parametru na pozostałe stopnie swobody układu.',
                        m_RC:'Zmiana masy napędy RC w sposób zauważalny wpłynęła na wartość drgań i prędkość kątową napędu. Przemieszczenia kątowe nie przyjmują wartości mniejszej niż {varphi_RC(t)_min} oraz większej niż {varphi_RC(t)_max}. Dla prędkości kątowej napędu minimalna wartość amplitudy to {Derivative(varphi_RC(t), t)_min}, a największą osiąganą wartością jest {Derivative(varphi_RC(t), t)_min}. Mniej istotne zmiany zaobserwowano dla parametru \({x(t)}\) oraz odpowiadającej temu przemieszczniu prędkości. Nie zauważono kluczowych zmian pozostałych zmiennych układu. ',
                        F:'Zaobserwowano istotny wpływ tego parametru na maksymalną wartość pokonanej drogi oraz osiąganą wartość prędkości, gdzie zaobserwować można liniową zależność. Maksymalne wartośći dla wymienionych współrzędnych przyjmują odpowiednio {x(t)_max} oraz {Derivative(x(t), t)_max} dla maksymalnej wartości siły. Zmianę dynamiki obserwuje się również dla \({varphi_RC(t)}\), gdzie największa wartość amplitudy to {varphi_RC(t)_max}, a najmniejsza {varphi_RC(t)_min}. ',
                        l_RC:'Zaobserwowano wpływ rozpatrywanego parametru na wartość drgań i prędkość kątową napędu RC. Przemieszczenia kątowe nie przyjmują wartości mniejszej niż {varphi_RC(t)_min} oraz większej niż {varphi_RC(t)_max}. Dla prędkości kątowej napędu minimalna wartość amplitudy to {Derivative(varphi_RC(t), t)_min}, a największą osiąganą wartością jest {Derivative(varphi_RC(t), t)_max}. Nie odmotowano natomiast istotnego wpływu tego parametru na pozostałe stopnie swobody układu. ',
                        m_3:'Zmiana masy wózka w sposób zauważalny wpłynęła na wartość drgań i prędkość kątową napędu. Przemieszczenia kątowe nie przyjmują wartości mniejszej niż {varphi_RC(t)_min} oraz większej niż {varphi_RC(t)_max}. Dla prędkości kątowej napędu minimalna wartość amplitudy to {Derivative(varphi_RC(t), t)_min}, a największą osiąganą wartością jest {Derivative(varphi_RC(t), t)_min}. Istotme zmiany zaobserwowano również dla parametru \({x(t)}\) - pokonanej drogi oraz \({Derivative(x(t), t)}\) - prędkości, gdzie zależność przedstawia się funkcją malejąca dla coraz większej masy wózka. Nie zauważono kluczowych zmian pozostałych zmiennych układu. ',
                        k_r:'Zaobserwowano wpływ rozpatrywanego parametru na wartość drgań i prędkość kątową napędu RC. Przemieszczenia kątowe nie przyjmują wartości mniejszej niż {varphi_RC(t)_min} oraz większej niż {varphi_RC(t)_max}. Dla prędkości kątowej napędu minimalna wartość amplitudy to {Derivative(varphi_RC(t), t)_min}, a największą osiąganą wartością jest {Derivative(varphi_RC(t), t)_max}. Nie odmotowano natomiast istotnego wpływu tego parametru na pozostałe stopnie swobody układu. '
                        }
        
        return self





summary_dict={time_param:time_domain_summary_param,
              time_x:time_domain_summary_x,
              time_varphi:time_domain_summary_varphi
              time_z:time_domain_summary_z,
              time_theta:time_domain_summary_theta,
              time_z_wrc:time_domain_summary_z_wrc,
              time_z_r:time_domain_summary_z_r,
              time_z_fr:time_domain_summary_z_fr,
              time_varphi_RC:time_domain_summary_varphi_RC,
              time_tval:time_domain_summary_tval,
              freq:freq_domain_summary,
              opti:summary_bank_optimization
             }


summary_bank_composed_model=[str(RandomDescription(
        time_domain_summary_param,
        time_domain_summary_x,
        time_domain_summary_varphi,
        time_domain_summary_z,
        time_domain_summary_varphi_RC,
            )) for obj in range(30)  ]    

summary_bank_chair_model=[str(RandomDescription(
        time_domain_summary_param,
        time_domain_summary_x,
        time_domain_summary_varphi,
        time_domain_summary_z,
            )) for obj in range(30)  ]   


summary_bank_drive_model=[str(RandomDescription(
        time_domain_summary_param,
        time_domain_summary_varphi_RC,
            )) for obj in range(30)  ]      


introduction_dict={gen1:sec_introduction_bank_gen1,
                  gen2:sec_introduction_bank_gen2,
                  gen3:sec_introduction_bank_gen3
                  }

sec_intro_composed_model=[str(RandomDescription(sec_introduction_bank_gen1,sec_introduction_bank_gen2,sec_introduction_bank_gen3)) for obj in range(30)  ]  


refs_dict={fig_no:introduction_bank_fig_no,
          given_data:introduction_bank_given_data}

intro_bank_composed_model=[str(RandomDescription(introduction_bank_fig_no,introduction_bank_given_data,introduction_bank_gen1)) for obj in range(30)  ]
intro_bank_chair_model=[str(RandomDescription(introduction_bank_fig_no,introduction_bank_given_data,introduction_bank_gen1)) for obj in range(30)  ]
intro_bank_drive_model=[str(RandomDescription(introduction_bank_fig_no,introduction_bank_given_data,introduction_bank_gen1)) for obj in range(30)  ]


conclusion_bank_composed_model=[str(RandomDescription( conclusion_bank_varphi_rc,conclusion_bank_x ,conclusion_bank_z,conclusion_bank_phi,conclusion_bank_no_impact)) for obj in range(30)  ]
conclusion_bank_chair_model=[str(RandomDescription(conclusion_bank_x,conclusion_bank_z,conclusion_bank_phi,conclusion_bank_no_impact)) for obj in range(30)  ]
conclusion_bank_drive_model=[str(RandomDescription(conclusion_bank_varphi_rc,conclusion_bank_no_impact)) for obj in range(30)  ]
