"""
This module provides basic tools for analysis of dynamic system (described by time-depended functions)
"""

from .dynamics import (LagrangesDynamicSystem, LinearDynamicSystem,
                       HarmonicOscillator, WeakNonlinearOscillator, DampedHarmonicOscillator)

from .continuous import ContinuousSystem